/**
 * MARCReader.java Version 0.1, October 2001
 *
 * Copyright (C) 2001  Bas Peters (mail@bpeters.com)
 *
 * This file is part of James (Java MARC Events).
 *
 * James is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * James is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with James; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **/

package com.bpeters.james;

import java.io.InputStream;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.FileNotFoundException;

import com.bpeters.james.datatype.*;

public class MARCReader {

    // record terminator
    private static final char RT = '\035';
    // field terminator
    private static final char FT = '\036';
    // delimiter (unit separator)
    private static final char US = '\037';

    MARCHandler marcHandler;
    ErrorHandler errorHandler;

    public void setMARCHandler(MARCHandler marcHandler) {
	this.marcHandler = marcHandler;
    }

    public void setErrorHandler(ErrorHandler errorHandler) {
	this.errorHandler = errorHandler;
    }

    public void parse(String filename) throws MARCException {
	try {
	    parseTape(new BufferedReader(new FileReader(filename)));
	} catch (FileNotFoundException e) {
	    throw new MARCException(e.toString());
	}
    }

    public void parse(InputStream src) throws MARCException {
	parseTape(new BufferedReader(new InputStreamReader(src)));
    }

    private void parseTape(BufferedReader in) throws MARCException {
	int counter = 0;
	boolean isFinished = false;

	// Fatal error notification if input stream contains no data
        if (in == null) {
            if(errorHandler != null)
		errorHandler.fatalError(new MARCException(
                    "input stream contains no data"));
	    return;
	}

	try {
	    // Start of tape notification
	    if (marcHandler != null)
                marcHandler.startTape();

	    // Read tape and process each record
	    while (! isFinished) {
		StringBuffer sb = new StringBuffer();
		while (true) {
		    int i = in.read();
		    if (i == -1) {
			isFinished = true;
			break;
		    } else if(i == RT) {
			sb.append((char)i);
			counter++;
			int retcode = parseRecord(counter,
                            sb.toString());

			// Message notification if return code = 0
			if (errorHandler != null && retcode == 0)
			    errorHandler.message(new MARCException(
                                counter +  ":OK"));
			break;

		    } else {
			sb.append((char)i);
		    }
		}
	    }

	    // End of tape notification
	    if (marcHandler != null)
                marcHandler.endTape();

	} catch (IOException e) {
	    throw new MARCException(e.toString());
	}
    }

    private int parseRecord(int counter, String record)
        throws MARCException {
	int recordLength = Numeric.getValue(record.substring(0, 5));
	int baseAddress = Numeric.getValue(record.substring(12, 17));

	// validate record by checking if the final character
        // is a record terminator and if the length of the
        // record string equals the record length in the leader.
	// If the record is not valid the method returns 1.
	if (record.charAt(recordLength -1) != RT ||
	    recordLength != record.length()) {
	    if (errorHandler != null)
		errorHandler.error(new MARCException(counter +
                    ":invalid record"));
	    return 1;
	}

	// directory and directory length
	String dir = record.substring(24, (baseAddress -1));
        int dirLength = dir.length();

	// validate the directory by checking if the final
	// character is a field terminator and if the modulo
        // of the directory length divided by 12 equals 0.
	// If the directory is not valid the method returns 1.
	if (//record.charAt(baseAddress -1) != FT ||
	    (dirLength == 0) || ((dirLength % 12) != 0)) {
	    if (errorHandler != null)
		errorHandler.error(new MARCException(counter +
                    ":invalid directory"));
	    return 1;
	}

	// create leader object
	Leader leader = new Leader();
	leader.setRecordLength(record.substring(0, 5));
	leader.setRecordStatus(record.charAt(5));
	leader.setTypeOfRecord(record.charAt(6));
	leader.setImplDefined1(record.substring(7,9).toCharArray());
	leader.setCharCodingScheme(record.charAt(9));
	leader.setIndicatorCount(record.charAt(10));
	leader.setSubfieldCodeLength(record.charAt(11));
	leader.setBaseAddressOfData(record.substring(12, 17));
	leader.setImplDefined2(record.substring(17, 20).toCharArray());
	leader.setEntryMap(record.substring(20, 24).toCharArray());

	// Start of record notification
	if (marcHandler != null)
	    marcHandler.startRecord(leader);

	int entries = dirLength / 12;
	for (int i=0; i<entries; i++) {

	    String tag = dir.substring(i*12, i*12 + 3);
	    // Tag validation
	    if (! Tag.isValid(tag)) {
		if (errorHandler != null)
		errorHandler.warning(new MARCException(
                    counter + ":" + tag + ":invalid tag"));
	    }

	    // Field length and starting position
	    String fieldLength = dir.substring(i*12 + 3, i*12 + 7);
	    String fieldStart = dir.substring(i*12 + 7, i*12 + 12);

	    if(! Numeric.isValid(fieldLength))
		errorHandler.warning(new MARCException(counter +
                    ":" + tag + ":invalid directory entry"));

	    if(! Numeric.isValid(fieldStart))
		errorHandler.warning(new MARCException(counter +
                    ":" + tag + ":invalid directory entry"));

	    int length = Numeric.getValue(fieldLength);
	    int start = Numeric.getValue(fieldStart);

	    // Get field data
	    String field = record.substring(
                (start + baseAddress),
	        (start + baseAddress + length - 1));

	    // Validate field by length
	    if (field.length() == 0) {
		errorHandler.error(new MARCException(counter +
                    ":" + tag + ":field contains no data"));
		break;
	    }

	    // Control field notification
	    if (Tag.isControlField(tag)) {
		if (marcHandler != null)
		    marcHandler.controlField(tag,
                        field.toCharArray());
	    } else {
		char ind1 = field.charAt(0);
		char ind2 = field.charAt(1);

		// Validate indicator value for ind1
		if (! Indicator.isValid(ind1))
		    if (errorHandler != null)
		        errorHandler.warning(new MARCException(counter +
			    ":" + tag + ":invalid indicator: " + ind1));

		// Validate indicator value for ind2
		if (! Indicator.isValid(ind2))
		    if (errorHandler != null)
		        errorHandler.warning(new MARCException(counter +
			    ":" + tag + ":invalid indicator: " + ind2));

		// Start of data field notification
		if (marcHandler != null)
		    marcHandler.startDataField(tag,
                        field.charAt(0), field.charAt(1));

		// Process data field
		parseDataField(counter, tag, field.substring(3));

		// End of data field notification
		if (marcHandler != null)
		    marcHandler.endDataField(tag);
	    }
	}
	// End of record notification
	if (marcHandler != null)
	    marcHandler.endRecord();

	// Return 0 if record succesfully parsed
	return 0;
    }

    private void parseDataField(int counter, String tag, String dataField)
        throws MARCException {
	boolean isFinished = false;
	int i = 0;
	while(! isFinished) {
	    StringBuffer sb = new StringBuffer();
	    char c;
	    while (true) {
		c = dataField.charAt(i);
		if (c == US) break;
		else sb.append(c);
		i++;
		if (i == dataField.length()) {
		    isFinished = true;
		    break;
		}
	    }

	    String subfield = sb.toString();
	    char code = subfield.charAt(0);

	    // Validate subfield delimiter value
	    if (! SubfieldCode.isValid(code))
		if (errorHandler != null)
		    errorHandler.warning(new MARCException(counter + ":" +
                        tag + ":invalid subfield code: " + code));

	    // Subfield notification
	    char[] data = subfield.substring(1).toCharArray();
	    if (marcHandler != null)
		marcHandler.subfield(code, data);
	    i++;
	}
    }

}

// End of MARCREader.java
