/**
 * XMLPrinter.java Version 0.1, October 2001
 *
 * Copyright (C) 2001  Bas Peters (mail@bpeters.com)
 *
 * This file is part of James (Java MARC Events).
 *
 * James is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * James is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with James; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **/

package com.bpeters.samples;

import java.io.FileOutputStream;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.Attribute;
import org.jdom.output.XMLOutputter;
import com.bpeters.james.*;

public class XMLPrinter {

    public static void main(String args[]) {
	if (args.length < 1) {
	    System.out.println("Usage: Driver infile [outfile]");
	    return;
	}
	// the first command line argument contains the inputfile name
	String infile = args[0];

	// Register the MARC reader class
	MARCReader mr = new MARCReader();

	// Check if an outputfile name is given
	if (args.length == 2) {
	    mr.setMARCHandler(new MARCHandlerXPImpl(args[1]));
	} else {
	    mr.setMARCHandler(new MARCHandlerXPImpl());
	}

	// Register the error handler class
        mr.setErrorHandler(new ErrorHandlerXPImpl());

	// Send the file to the parse method
	try {
	    mr.parse(infile);
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

}

class MARCHandlerXPImpl implements MARCHandler {

    Document doc;
    Element root;
    Element record;
    Element dataField;
    String filename;

    public MARCHandlerXPImpl() {}

    public MARCHandlerXPImpl(String filename) {
	this.filename = filename;
    }

    public void startTape() {
	doc = new Document(new Element("catalog"));
	root = doc.getRootElement();
    }

    public void endTape() {
	try {
	    XMLOutputter serializer = new XMLOutputter("  ", true);
	    if (filename != null) {
		FileOutputStream out = new FileOutputStream(filename);
		serializer.output(doc, out);
		out.flush();
		out.close();
	    } else {
		serializer.output(doc, System.out);
	    }
	} catch (Exception e) {
	    e.printStackTrace();
	}
    }

    public void startRecord(Leader leader) {

	// Create a new record element
	record = new Element("record");

	// Create a leader element
	Element ldr = new Element("leader");

	// Add child elements to the leader using
	// the LeaderChild class
	ldr.addContent(new LeaderChild("recordLength",
	    leader.getRecordLength()));

	ldr.addContent(new LeaderChild("recordStatus",
	    String.valueOf(leader.getRecordStatus())));

	ldr.addContent(new LeaderChild("typeOfRecord",
	    String.valueOf(leader.getTypeOfRecord())));

	ldr.addContent(new LeaderChild("implDefined1",
	    new String(leader.getImplDefined1())));

	ldr.addContent(new LeaderChild("charCodingScheme",
	    String.valueOf(leader.getCharCodingScheme())));

	ldr.addContent(new LeaderChild("indicatorCount",
	    String.valueOf(leader.getIndicatorCount())));

	ldr.addContent(new LeaderChild("subfieldCodeLength",
	    String.valueOf(leader.getSubfieldCodeLength())));

	ldr.addContent(new LeaderChild("baseAddressOfData",
	    leader.getBaseAddressOfData()));

	ldr.addContent(new LeaderChild("implDefined2",
	    new String(leader.getImplDefined2())));

	ldr.addContent(new LeaderChild("entryMap",
	    new String(leader.getEntryMap())));

	// Add the leader to the record
	record.addContent(ldr);
    }

    public void endRecord() {
	root.addContent(record);
    }

    public void controlField(String tag, char[] chars) {
	Element controlField = new Element("controlField")
	    .setAttribute("tag", tag)
	    .setAttribute("value", new String(chars));
	record.addContent(controlField);
    }

    public void startDataField(String tag, char ind1, char ind2) {
	dataField = new Element("dataField")
	    .setAttribute("tag", tag)
	    .setAttribute("ind1", String.valueOf(ind1))
	    .setAttribute("ind2", String.valueOf(ind2));
    }

    public void endDataField(String tag) {
	record.addContent(dataField);
    }

    public void subfield(char code, char[] chars) {
	String value = new String(chars);
	Element subfield = new Element("subfield")
	    .setAttribute("code", String.valueOf(code))
	    .addContent(value);
	dataField.addContent(subfield);
    }

}

class ErrorHandlerXPImpl implements ErrorHandler {

    public void error(MARCException e) {
	System.out.println("Error: " + e.getMessage());
    }

    public void warning(MARCException e) {
	System.out.println("Warning: " + e.getMessage());
    }

    public void message(MARCException e) {
	System.out.println("Message: " + e.getMessage());
    }

    public void fatalError(MARCException e) {
	System.out.println("Fatal error: " + e.getMessage());
    }

}

class LeaderChild extends Element {

    public LeaderChild(String element, String value) {
	super(element);
	setAttribute("value", value);
    }
}

// End of XMLPrinter.java
